import java.io.*;
import java.net.*;
import java.util.Enumeration;
import java.util.NoSuchElementException;

// Class loader for loading bytes codes from a file.
public class FlatClassLoader extends ClassLoader {
    String path;

    public FlatClassLoader(String path) {
        this.path = path;
    }

    // Loads the bytes from file 
    byte[] getBytecodes(String classname) throws ClassNotFoundException {
        // To get file name, remove the package name, if any.
        String filename;
        if (classname.indexOf(".") >= 0) {
            filename = classname.substring(classname.lastIndexOf(".")+1);
        } else {
            filename = classname;
        }
        // Make sure the filename ends with .class
        filename += ".class";

        File fullname = new File(path, filename);
        System.out.println("class file name: " + fullname);
        try {
            // Read in the byte codes.
            InputStream is = new FileInputStream(fullname);
            int bufsize = (int)fullname.length();
            byte buf[] = new byte[bufsize];
            is.read(buf, 0, bufsize);
            is.close();

            return buf;
        } catch (Exception e) {
            throw new ClassNotFoundException(classname);
        }
    }

    protected synchronized Class findClass(String className) 
        throws ClassNotFoundException {
            byte[] buf = getBytecodes(className);

            // Check if package needs to be defined
            if (className.indexOf('.') != -1) {
                String pkg = className.substring(0, className.lastIndexOf('.'));
                if (getPackage(pkg) == null) {
                    definePackage(pkg, null, null, null, null, null, null, null);
                }
            }

            return defineClass(className, buf, 0, buf.length);
        }

    // Returns a URL containing the location of the named resource.
    protected URL findResource(String name) {
        try {
            File file = new File(path, name);
            String absPath = 
                file.getAbsolutePath().replace(file.separatorChar, '/');

            // If leading character is not '/', add for URL.
            if (absPath.charAt(0) != '/') {
                absPath = '/' + absPath;
            }

            System.out.println("resource name: " + absPath);
            return new URL("file:" + absPath);
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
        return null;
    }

    // Returns an enumeration containing URLs of the named resource.
    protected Enumeration findResources(String name) throws IOException {
        // We only look in one place;
        return new SingleEnum(findResource(name));
    }

    private static class SingleEnum implements Enumeration {
        Object element;
        SingleEnum(Object element) {
            this.element = element;
        }
        public boolean hasMoreElements() {
            return element == null;
        }

        public Object nextElement() {
            if (element == null) {
                throw new NoSuchElementException();
            } else {
                Object answer = element;
                element = null;
                return answer;
            }
        }
    }
}
