import java.text.AttributedString;
import java.text.AttributedCharacterIterator;
import java.text.Annotation;
import java.util.Locale;

class Main {
    // Assign READING constant to a variable (to save typing).
    static AttributedCharacterIterator.Attribute attrREAD =
        AttributedCharacterIterator.Attribute.READING;

    public static void main(String[] args) {
        // Create an attributed string.
        AttributedString attrStr = new AttributedString("Vous l'aimez.");

        // Add reading attribute key/value pairs for each subrange.
        attrStr.addAttribute(attrREAD, new Annotation("vu"), 0, 4);   // Vous
        attrStr.addAttribute(attrREAD, new Annotation("l"), 5, 7);    // l'
        attrStr.addAttribute(attrREAD, new Annotation("eme"), 7, 12); // aimez

        AttributedCharacterIterator.Attribute[] matchAttr =
            new AttributedCharacterIterator.Attribute[] {
                attrREAD
            };

        // Create an iterator for the attributed string.
        AttributedCharacterIterator iter = 
           attrStr.getIterator(matchAttr, 0, 10);

        // Print each run of the READING attribute.
        while (iter.current() != AttributedCharacterIterator.DONE) {
            printToRunLimit(iter);
        }
    }

    // Print the run start, run limit, string and attribute.
    public static void printToRunLimit(AttributedCharacterIterator aci) {
        // Get the run limit of the READING attribute.
        int runStart = aci.getRunStart(attrREAD);
        printInColumn("" + runStart, 5);
        int runLimit = aci.getRunLimit(attrREAD);
        printInColumn("" + runLimit, 5);
        Object attr = aci.getAttribute(attrREAD);

        // Print characters to the run limit
        System.out.print("\"");
        while (aci.getIndex() < runLimit) {
            System.out.print(aci.current());
            aci.next();
        }
        System.out.print("\"   ");
        printInColumn("", 10 - (runLimit - runStart));
        if (attr != null) {
            if (attr.getClass() == Annotation.class) {
                System.out.println(((Annotation)attr).getValue());
            }
        } else {
            System.out.println(attr);
        }
    }

    // Print string, then pad spaces to a particular vertical column
    static void printInColumn(String str, int col) {
        System.out.print(str);
        for (int p = str.length(); p < col; ++p) {
            System.out.print(" ");
        }
    }
}
