import java.text.AttributedCharacterIterator.Attribute;
import java.util.Map;
import java.util.HashMap;
import java.io.InvalidObjectException;

public final class MyTextAttribute extends Attribute {
    // Table of all attribute constants in this class, used by readResolve
    private static final Map instanceMap = new HashMap(2);

    // Constructs an instance of MyTextAttribute with the given name.
    protected MyTextAttribute(String name) {
        super(name);
        if (this.getClass() == MyTextAttribute.class) {
            instanceMap.put(name, this);
        }
    }

    // Resolves instances being deserialized to the predefined constants.
    protected Object readResolve() throws InvalidObjectException {
        if (this.getClass() != MyTextAttribute.class) {
            throw new InvalidObjectException(
                "subclass didn't correctly implement readResolve");
        }
        
        MyTextAttribute instance = 
                            (MyTextAttribute) instanceMap.get(getName());
        if (instance != null) {
            return instance;
        } else {
            throw new InvalidObjectException("unknown attribute name");
        };
    }

    // Attribute key for glossary definition
    public static final MyTextAttribute GLOSSARY_DEFINITION = 
        new MyTextAttribute("glossary definition");

    // Attribute key for syllable breaks
    public static final MyTextAttribute SYLLABLES = 
        new MyTextAttribute("syllables");
}
